package wechat

import (
	"bytes"
	"encoding/json"
	"io/ioutil"
	"net/http"
)

type WeChatClient struct {
	AppId          string
	Token          string
	EncodingAESKey string
	Appsecret      string
	ApiUrl         string
}

func NewWeChatClient(AppId, Token, EncodingAESKey, Appsecret, ApiUrl string) *WeChatClient {

	return &WeChatClient{
		AppId:          AppId,          //"you AppId",
		Token:          Token,          // "you Token",
		EncodingAESKey: EncodingAESKey, //"you EncodingAESKey",
		Appsecret:      Appsecret,      //"you Appsecret",
		ApiUrl:         ApiUrl,         // "you ApiUrl",
	}
}

//获取预授权码
func (client *WeChatClient) GetPreAuthCode(access_token string) (pre_auth_code string, err error) {
	url := "https://api.weixin.qq.com/cgi-bin/component/api_create_preauthcode?component_access_token=" + access_token
	params := map[string]interface{}{
		"component_appid": client.AppId,
	}
	res, err := client.PostJson(url, params)
	if err != nil {
		return
	}
	var getPreAuthCode *GetPreAuthCode
	err = json.Unmarshal([]byte(res), &getPreAuthCode)
	if err != nil {
		return
	}
	pre_auth_code = getPreAuthCode.PreAuthCode
	return
}

//获取令牌
func (client *WeChatClient) GetComponentAccessToken(component_verify_ticket string) (res *ApiComponentToken, err error) {
	url := "https://api.weixin.qq.com/cgi-bin/component/api_component_token"
	params := map[string]interface{}{
		"component_appid":         client.AppId,
		"component_appsecret":     client.Appsecret,
		"component_verify_ticket": component_verify_ticket,
	}
	str, err := client.PostJson(url, params)
	if err != nil {
		return
	}

	err = json.Unmarshal([]byte(str), &res)
	if err != nil {
		return
	}
	return
}

func (client *WeChatClient) PostJson(urlMethod string, params map[string]interface{}) (str string, err error) {
	postbyte, _ := json.Marshal(params)

	req, err := http.NewRequest("POST", urlMethod, bytes.NewReader(postbyte))
	postString := string(postbyte)
	if checkErr(err, "wechat NewRequest request"+urlMethod+"myParam:【"+postString+"】") {
		return
	}
	req.Header.Set("Content-Type", "application/json;charset=UTF-8")
	resp, err := (&http.Client{}).Do(req)
	if checkErr(err, "wechat NewRequest do request"+urlMethod+"myParam:【"+postString+"】") {
		return
	}
	body, _ := ioutil.ReadAll(resp.Body)
	resp.Body.Close()
	str = string(body)

	//	glog.Path("./logs/wechatRequire").File("info-{Ymd}.log").Printf(urlMethod + "myParam:【" + postString + "】 thirdParam：【" + str + "】")
	return
}

////错误处理函数
func checkErr(err error, extra string) bool {
	if err != nil {
		formatStr := " Err : %s\n"
		if extra != "" {
			formatStr = extra + formatStr
		}
		//	glog.Path("./logs/wechatRequire").File("error-{Ymd}.log").Printf("formatStr %s  error %v", formatStr, err)
		return true
	}
	return false
}

//绑定公众号
func (client *WeChatClient) Binding(authorizer_appid string, authorizer_access_token string) (res *BindingDate, err error) {
	url := "https://api.weixin.qq.com/cgi-bin/open/create?access_token=" + authorizer_access_token
	if err != nil {
		return
	}
	params := map[string]interface{}{
		"appid": authorizer_appid,
	}
	str, err := client.PostJson(url, params)
	if err != nil {
		return
	}
	err = json.Unmarshal([]byte(str), &res)
	return
}

//获取授权信息
func (client *WeChatClient) AuthorizationData(auth_code, access_token string) (res *AuthorizationDataRes, err error) {
	url := "https://api.weixin.qq.com/cgi-bin/component/api_query_auth?component_access_token=" + access_token
	if err != nil {
		return
	}
	params := map[string]interface{}{
		"component_appid":    client.AppId,
		"authorization_code": auth_code,
	}
	str, err := client.PostJson(url, params)
	if err != nil {
		return
	}
	err = json.Unmarshal([]byte(str), &res)
	return
}

//刷新token
func (client *WeChatClient) AuthorizerToken(authorizer_appid, access_token, redisResult string) (res *AuthorizerTokenRes, err error) {

	url := "https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?component_access_token=" + access_token
	params := map[string]interface{}{
		"authorizer_appid":         authorizer_appid,
		"component_appid":          client.AppId,
		"authorizer_refresh_token": redisResult,
	}
	str, err := client.PostJson(url, params)
	if err != nil {
		return
	}
	err = json.Unmarshal([]byte(str), &res)
	return
}
