package wechat

import (
	"encoding/xml"
	"time"
)

type ApiComponentToken struct {
	ComponentAccessToken string `json:"component_access_token"`
	ExpiresIn            int    `json:"expires_in"`
}

type GetPreAuthCode struct {
	PreAuthCode string `json:"pre_auth_code"`
	ExpiresIn   int    `json:"expires_in"`
}

type CheckAuth struct {
	ErrCode int    `json:"errcode"`
	ErrMsg  string `json:"errmsg"`
	Data    *Auth  `json:"data"`
}
type Auth struct {
	AppId           string `json:"appid"`
	ServiceId       int    `json:"service_id"`
	SpecificationId string `json:"specification_id"`
}

type GetList struct {
	ErrCode     int            `json:"errcode"`
	ErrMsg      string         `json:"errmsg"`
	ServiceList []*ServiceList `json:"service_list"`
}

type ServiceList struct {
	ServiceId   int    `json:"service_id"`
	ServiceName string `json:"service_name"`
	ExpireTime  string `json:"expire_time"`
	ServiceType int    `json:"service_type"`
}

type GetAuthorizerInfo struct {
	AuthorizationInfo *AuthorizationInfo `json:"authorization_info"`
	AuthorizerInfo    *AuthorizerInfo    `json:"authorizer_info"`
}

type AuthorizerInfo struct {
	NickName      string `json:"nick_name"`
	UserName      string `json:"user_name"`
	PrincipalName string `json:"principal_name"`
}

type AuthorizationInfo struct {
	AuthorizationAppid     string `json:"authorization_appid"`
	AuthorizerRefreshToken string `json:"authorizer_refresh_token"`
}

type ApiAuthorizerToken struct {
	AuthorizerAccessToken  string `json:"authorizer_access_token"`
	ExpiresIn              int    `json:"expires_in"`
	AuthorizerRefreshToken string `json:"authorizer_refresh_token"`
}

type GetAuthorizerList struct {
	TotalCount int                   `json:"total_count"`
	List       []*AuthorizerListInfo `json:"list"`
}

type AuthorizerListInfo struct {
	AuthorizerAppid string `json:"authorizer_appid"`
	RefreshToken    string `json:"refresh_token"`
	AuthTime        int    `json:"auth_time"`
}

type GetCategory struct {
	ErrCode int         `json:"errcode"`
	ErrMsg  string      `json:"errmsg"`
	CatList []*Category `json:"cat_list"`
}

type Category struct {
	CatId  int    `json:"cat_id"`
	FCatId int    `json:"f_cat_id"`
	Name   string `json:"name"`
}

type GetBrand struct {
	ErrCode int       `json:"errcode"`
	ErrMsg  string    `json:"errmsg"`
	Brands  []*Brands `json:"brands"`
}

type Brands struct {
	FirstCatId  int        `json:"first_cat_id"`
	SecondCatId int        `json:"second_cat_id"`
	ThirdCatId  int        `json:"third_cat_id"`
	BrandInfo   *BrandInfo `json:"brand_info"`
}

type BrandInfo struct {
	BrandId   int    `json:"brand_id"`
	BrandName string `json:"brand_name"`
}

type GetFreightTemplate struct {
	ErrCode      int             `json:"errcode"`
	ErrMsg       string          `json:"errmsg"`
	TemplateList []*TemplateInfo `json:"template_list"`
}

type TemplateInfo struct {
	TemplateId    int    `json:"template_id"`
	Name          string `json:"name"`
	ValuationType int    `json:"valuation_type"`
}

type GetShopCat struct {
	ErrCode     int        `json:"errcode"`
	ErrMsg      string     `json:"errmsg"`
	ShopcatList []*Shopcat `json:"shopcat_list"`
}

type Shopcat struct {
	ShopcatId   int    `json:"shopcat_id"`
	ShopcatName string `json:"shopcat_name"`
	FShopcatId  int    `json:"f_shopcat_id"`
	CatLevel    int    `json:"cat_level"`
}

type AddSpuRes struct {
	ErrCode int            `json:"errcode"`
	ErrMsg  string         `json:"errmsg"`
	Data    *AddSpuResData `json:"data"`
}

type AddSpuResData struct {
	ProductId    int    `json:"product_id"`
	OutProductId string `json:"out_product_id"`
	CreateTime   string `json:"create_time"`
}

type UpdateSpuRes struct {
	ErrCode int               `json:"errcode"`
	ErrMsg  string            `json:"errmsg"`
	Data    *UpdateSpuResData `json:"data"`
}

type UpdateSpuResData struct {
	ProductId    int    `json:"product_id"`
	OutProductId string `json:"out_product_id"`
	UpdateTime   string `json:"update_time"`
}

type DelSpuRes struct {
	ErrCode int    `json:"errcode"`
	ErrMsg  string `json:"errmsg"`
}
type SpuOnSale struct {
	ErrCode int    `json:"errcode"`
	ErrMsg  string `json:"errmsg"`
}

type SpuDownSale struct {
	ErrCode int    `json:"errcode"`
	ErrMsg  string `json:"errmsg"`
}

type GetSpu struct {
	ErrCode int      `json:"errcode"`
	ErrMsg  string   `json:"errmsg"`
	Data    *SpuInfo `json:"data"`
}
type GetSpuList struct {
	ErrCode int        `json:"errcode"`
	ErrMsg  string     `json:"errmsg"`
	Spus    []*SpuInfo `json:"spus"`
}

type SpuInfo struct {
	Title    string   `json:"title"`
	SubTitle string   `json:"sub_title"`
	HeadImg  []string `json:"head_img"`
	DescInfo struct {
		Imgs []string `json:"imgs"`
	} `json:"desc_info"`
	OutProductId string `json:"out_product_id"`
	ProductId    int    `json:"product_id"`
	BrandId      int    `json:"brand_id"`
	Status       int    `json:"status"`
	EditStatus   int    `json:"edit_status"`
}

type GetSku struct {
	ErrCode int      `json:"errcode"`
	ErrMsg  string   `json:"errmsg"`
	Data    *SkuInfo `json:"data"`
}

type GetSkuList struct {
	ErrCode int        `json:"errcode"`
	ErrMsg  string     `json:"errmsg"`
	Skus    []*SkuInfo `json:"skus"`
}

type SkuInfo struct {
	ProductId    int        `json:"product_id"`
	OutProductId string     `json:"out_product_id"`
	OutSkuId     string     `json:"out_sku_id"`
	SkuId        int        `json:"sku_id"`
	ThumbImg     string     `json:"thumb_img"`
	SalePrice    int        `json:"sale_price"`
	MarketPrice  int        `json:"market_price"`
	StockNum     int        `json:"stock_num"`
	SkuCode      string     `json:"sku_code"`
	Code         string     `json:"code"`
	Barcode      string     `json:"barcode"`
	SkuAttrs     []*SkuAttr `json:"sku_attrs"`
}

type SkuAttr struct {
	AttrKey   string `json:"attr_key"`
	AttrValue string `json:"attr_value"`
}

type DelSku struct {
	ErrCode int    `json:"errcode"`
	ErrMsg  string `json:"errmsg"`
}

type AddSkuBatch struct {
	ErrCode int                `json:"errcode"`
	ErrMsg  string             `json:"errmsg"`
	Data    []*AddSkuBatchData `json:"data"`
}
type AddSkuBatchData struct {
	SkuId      int    `json:"sku_id"`
	OutSkuId   string `json:"out_sku_id"`
	CreateTime string `json:"create_time"`
}

type OrderGetList struct {
	ErrCode int              `json:"errcode"`
	ErrMsg  string           `json:"errmsg"`
	Orders  []*OrderListInfo `json:"orders"`
}

type OrderListInfo struct {
	OrderId     int          `json:"order_id"`
	Status      int          `json:"status"`
	CreateTime  string       `json:"create_time"`
	UpdateTime  string       `json:"update_time"`
	OrderDetail *OrderDetail `json:"order_detail"`
}

type OrderDetail struct {
	ProductInfos []*OrderDetailProductInfo `json:"product_infos"`
	PayInfo      *OrderDetailPayInfo       `json:"pay_info"`
	PriceInfo    *OrderDetailPriceInfo     `json:"price_info"`
	DeliveryInfo *DeliveryInfo             `json:"delivery_info"`
}

type OrderDetailProductInfo struct {
	ProductId             int        `json:"product_id"`
	SkuId                 int        `json:"sku_id"`
	SkuCnt                int        `json:"sku_cnt"`
	OnAftersaleSkuCnt     int        `json:"on_aftersale_sku_cnt"`
	FinishAftersaleSkuCnt int        `json:"finish_aftersale_sku_cnt"`
	Title                 string     `json:"title"`
	ThumbImg              string     `json:"thumb_img"`
	SkuAttrs              []*SkuAttr `json:"sku_attrs"`
	SalePrice             int        `json:"sale_price"`
}

type OrderDetailPayInfo struct {
	PayMethod     string `json:"pay_method"`
	PrepayId      string `json:"prepay_id"`
	TransactionId string `json:"transaction_id"`
	PrepayTime    string `json:"prepay_time"`
	PayTime       string `json:"pay_time"`
}

type OrderDetailPriceInfo struct {
	ProductPrice    int `json:"product_price"`
	OrderPrice      int `json:"order_price"`
	Freight         int `json:"freight"`
	DiscountedPrice int `json:"discounted_price"`
}

type DeliveryInfo struct {
	AddressInfo *AddressInfo `json:"address_info"`
}

type AddressInfo struct {
	UserName     string `json:"user_name"`
	ProvinceName string `json:"province_name"`
	CityName     string `json:"city_name"`
	CountyName   string `json:"county_name"`
	DetailInfo   string `json:"detail_info"`
	TelNumber    string `json:"tel_number"`
}

type GetCompanyList struct {
	ErrCode     int            `json:"errcode"`
	ErrMsg      string         `json:"errmsg"`
	CompanyList []*CompanyList `json:"company_list"`
}

type CompanyList struct {
	DeliveryId   string `json:"delivery_id"`
	DeliveryName string `json:"delivery_name"`
}

type DeliverySend struct {
	ErrCode int    `json:"errcode"`
	ErrMsg  string `json:"errmsg"`
}

type BindingDate struct {
	Errcode int    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
}
type AuthorizationDataRes struct {
	AuthorizationInfo struct {
		AuthorizerAppid        string `json:"authorizer_appid"`
		AuthorizerAccessToken  string `json:"authorizer_access_token"`
		ExpiresIn              int    `json:"expires_in"`
		AuthorizerRefreshToken string `json:"authorizer_refresh_token"`
		FuncInfo               []struct {
			FuncscopeCategory struct {
				ID int `json:"id"`
			} `json:"funcscope_category"`
		} `json:"func_info"`
	} `json:"authorization_info"`
}
type AuthorizerTokenRes struct {
	AuthorizerAccessToken  string `json:"authorizer_access_token"`
	ExpiresIn              int    `json:"expires_in"`
	AuthorizerRefreshToken string `json:"authorizer_refresh_token"`
}
type GetTicketRes struct {
	Ticket string `json:"ticket"`
	URL    string `json:"url"`
}

type PushWxMsgRes struct {
	Errcode int    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
	Msgid   int    `json:"msgid"`
}

type TextRequestBody struct {
	XMLName      xml.Name `xml:"xml"`
	ToUserName   string
	FromUserName string
	CreateTime   time.Duration
	MsgType      string
	Url          string
	PicUrl       string
	MediaId      string
	ThumbMediaId string
	Content      string
	MsgId        int
	Location_X   string
	Location_Y   string
	Label        string
	Event        string `xml:"Event"`
	EventKey     string `xml:"EventKey"`
	ScanCodeInfo struct {
		ScanType   string `xml:"ScanType"`
		ScanResult string `xml:"ScanResult"`
	} `xml:"ScanCodeInfo"`
}

type TextResponseBody struct {
	XMLName      xml.Name `xml:"xml"`
	ToUserName   CDATAText
	FromUserName CDATAText
	CreateTime   string
	MsgType      CDATAText
	Content      CDATAText
}

type EncryptRequestBody struct {
	XMLName    xml.Name `xml:"xml"`
	ToUserName string
	Encrypt    string
}

type EncryptResponseBody struct {
	XMLName      xml.Name `xml:"xml"`
	Encrypt      CDATAText
	MsgSignature CDATAText
	TimeStamp    string
	Nonce        CDATAText
}

type EncryptResponseBody1 struct {
	XMLName      xml.Name `xml:"xml"`
	Encrypt      string
	MsgSignature string
	TimeStamp    string
	Nonce        string
}

type CDATAText struct {
	Text string `xml:",innerxml"`
}
